'''
Created on 28/03/2010

@author: miguel
'''

import pygame
import random
import sys
import BugExit
import Tree
import BugManual, Bug
from vec2d import vec2d
from GameBoard import GameBoard
from Instruction import Instruction
from Box import Box
from EnemyPanel import EnemyPanel
from PlayerPanel import PlayerPanel
from Player import Player
from Computer import Computer
from InstructionsPanel import InstructionsPanel

class Game(object):
    ''' Main class of the game
    '''
    # Game parameters
    SCREEN_WIDTH = 800
    SCREEN_HIGH = 600
    BACKGROUND_IMAGE = './images/background.png'
    MUSIC_FILE = './images/country.mod'

    (PLAYER, COMPUTER, NOBODY) = range (3)

    INTRO_IMAGE1 = './images/intro_1.png'
    INTRO_IMAGE2 = './images/intro_2.png'
    MENU_IMAGE = './images/menu.png'
    END1_IMAGE = './images/end1.png'
    END2_IMAGE = './images/end2.png'


    def __init__(self):
        pygame.init()
        # pygame.key.set_repeat(10,10) repeticion de teclas, seguramente no hace falta

        self.screen = pygame.display.set_mode((self.SCREEN_WIDTH, self.SCREEN_HIGH),0,32)
        self.background_image = pygame.image.load(self.BACKGROUND_IMAGE)
        self.intro_image1 = pygame.image.load(self.INTRO_IMAGE1)
        self.intro_image2 = pygame.image.load(self.INTRO_IMAGE2)
        self.menu_image = pygame.image.load(self.MENU_IMAGE)
        self.end1_image = pygame.image.load(self.END1_IMAGE)
        self.end2_image = pygame.image.load(self.END2_IMAGE)

        self.clock = pygame.time.Clock()
        self.paused = False
        self.game_board = GameBoard()

        self.trees = pygame.sprite.Group()
        pygame.mixer.music.load(self.MUSIC_FILE)

        self.winner = self.NOBODY

    def __quit(self):
        sys.exit()

    def __draw(self):
        self.__draw_background()
        '''draw bug exits'''
        for bug_exit in self.player.bug_exits:
            bug_exit.draw()
        for bug_exit in self.computer.bug_exits:
            bug_exit.draw()

        '''draw boxes'''
        for user_box in self.player.user_boxes:
            user_box.draw()
        for com_box in self.computer.com_boxes:
            com_box.draw()

        '''draw bugs'''
        for user_bug in self.player.user_bugs:
            user_bug.draw()
        for com_bug in self.computer.com_bugs:
            com_bug.draw()

        '''draw panels'''
        self.player.player_panel.draw()
        self.computer.enemy_panel.draw()

        '''other'''
        for tree in self.trees:
            tree.draw()

        self.instructions_panel.draw()
        #for user_bug_manual in self.user_bug_manual:
        #    user_bug_manual.draw()

    def __draw_background(self):
        bg_rect = self.background_image.get_rect()
        self.screen.blit(self.background_image, bg_rect)

    def __create_obstacles(self):
        self.tree_estructure = [[(0, 210), (30, 210), (60, 210), (90, 210), (120, 210), (150, 210), (180, 210), (210, 210), (180, 300), (210, 300), (240, 300), (270, 300), (300, 300), (330, 300), (360, 300), (390, 300), (360, 390), (390, 390), (420, 390), (450, 390), (480, 390), (510, 390), (540, 390)],
                                [(0, 210), (30, 210), (60, 210), (90, 210), (120, 210), (150, 210), (180, 210), (210, 210), (240, 210), (270, 210), (300, 210), (330, 210), (360, 210), (390, 210), (420, 210), (450, 210), (480, 210), (510, 210), (540, 210),
                                 (0, 420), (30, 420), (60, 420), (90, 420), (120, 420), (150, 420), (180, 420), (210, 420), (240, 420), (270, 420), (300, 420), (330, 420), (360, 420), (390, 420), (420, 420), (450, 420), (480, 420), (510, 420), (540, 420)],
                                [(0, 450), (30, 450), (60, 450), (90, 450), (120, 360), (150, 360), (180, 360), (210, 360), (210, 300), (240, 300), (270, 300), (300, 300), (330, 300), (300, 240), (330, 240), (360, 240), (390, 240), (420, 150), (450, 150), (480, 150), (510, 150), (540, 150)]]

        self.estructure_selected = random.randint(0, len(self.tree_estructure) - 1)
        for object in self.tree_estructure[self.estructure_selected]:
            self.trees.add(Tree.Tree(self.screen, self, self.background_image.get_rect(), object))

        #self.trees.add(Tree.Tree(self.screen, self, self.background_image.get_rect(), (240, 300)))
        #self.trees.add(Tree.Tree(self.screen, self, self.background_image.get_rect(), (270, 300)))
        #self.trees.add(Tree.Tree(self.screen, self, self.background_image.get_rect(), (300, 300)))
        #self.trees.add(Tree.Tree(self.screen, self, self.background_image.get_rect(), (330, 300)))

    def explosion(self, position):
        '''calculate affected positions'''
        '''position double effect'''
        self.position0 = vec2d(position)
        '''positions single effect'''
        self.position1 = vec2d(position) + vec2d(-30, 0)
        self.position2 = vec2d(position) + vec2d(0, -30)
        self.position3 = vec2d(position) + vec2d(30, 0)
        self.position4 = vec2d(position) + vec2d(0, 30)
        self.position_list = [self.position0, self.position1, self.position2, self.position3, self.position4]

        for tree in self.trees:
            tree.hit_tree(self.position_list)

        for user_box in self.player.user_boxes:
            user_box.hit_box(self.position_list)

        for com_box in self.computer.com_boxes:
            com_box.hit_box(self.position_list)

        for user_bug in self.player.user_bugs:
            user_bug.hit_bug(self.position_list)

        for com_bug in self.computer.com_bugs:
            com_bug.hit_bug(self.position_list)

        pass

    def __load_other_elements(self):
        self.player = Player(self.screen, self, self.background_image.get_rect())
        self.computer = Computer(self.screen, self, self.background_image.get_rect())
        self.instructions_panel = InstructionsPanel(self.screen, self)


    def is_position_available(self, sprite):
        for tree in self.trees:
            if pygame.sprite.collide_rect(sprite, tree):
                return True
            else:
                pass
        return False

    def intro(self):
        self.intro = self.intro_image1
        bg_rect = self.intro_image1.get_rect()
        counter = 0
        skip_intro = False
        while True:
            time_passed = self.clock.tick(30)

            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.__quit()
                elif event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_ESCAPE or event.key == pygame.K_q:
                        skip_intro = True
            if skip_intro == True:
                break
            if counter > 200 and counter < 400:
                self.intro = self.intro_image2
            elif counter > 400:
                break
            counter = counter + 1
            self.screen.blit(self.intro, bg_rect)
            pygame.display.flip()

    def menu(self):
        bg_rect = self.menu_image.get_rect()

        skip_intro = False
        while True:
            time_passed = self.clock.tick(30)

            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.__quit()
                elif event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_s:
                        skip_intro = True
            if skip_intro == True:
                break

            self.screen.blit(self.menu_image, bg_rect)
            pygame.display.flip()

    def run2(self):
        self.intro()
        self.menu()
        self.start_game()

    def run_without_intro(self):
        self.menu()
        self.start_game()

    '''antiguo run'''
    def start_game(self):
        # The main loop of the game
        self.__create_obstacles()
        self.__load_other_elements()

        pygame.mixer.music.play()

        while True:
            if not pygame.mixer.music.get_busy():
                pygame.mixer.music.play()

            time_passed = self.clock.tick(30)
            # game pause; we don't want to jump forward later
            if time_passed > 100:
                continue

            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.__quit()
                elif event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_p:
                        self.paused = not self.paused
                    elif event.key == pygame.K_ESCAPE:
                        self.__quit()
                    #elif event.key == pygame.K_SPACE:
                    #    self.computer.enemy_panel.set_support_discovered()
                    #elif event.key == pygame.K_LEFT:
                    #    pass
                    #    for user_bug in self.user_bug_manual:
                    #        user_bug.go_left()
                    #elif event.key == pygame.K_RIGHT:
                    #    pass
                    #    for user_bug in self.user_bug_manual:
                    #        user_bug.go_right()
                    #elif event.key == pygame.K_UP:
                    #    for user_bug in self.user_bug_manual:
                    #        user_bug.go_up()
                    #elif event.key == pygame.K_DOWN:
                    #    for user_bug in self.user_bug_manual:
                    #        user_bug.go_down()
                #elif event.type == pygame.KEYUP:
                #    if event.key == pygame.K_LEFT:
                #        pass
                #    elif event.key == pygame.K_RIGHT:
                #        pass
                elif event.type == pygame.MOUSEBUTTONDOWN:
                    (left_button, middle_button, right_button) = pygame.mouse.get_pressed()
                    if left_button:
                        self.instructions_panel.click(pygame.mouse.get_pos())

            if not self.paused:
                self.computer.update(time_passed)
                self.player.update(time_passed)
                self.instructions_panel.update()

                self.__draw()

            pygame.display.flip()
            #self.game_board.print_board()
            #print '----------------------------------------'
            if self.winner != self.NOBODY:
                break

        if self.winner == self.PLAYER:
            pygame.mixer.music.stop()
            self.win_player()
        elif self.winner == self.COMPUTER:
            pygame.mixer.music.stop()
            self.win_computer()

    def win_player(self):
        self.end2 = self.end2_image
        bg_rect = self.end2_image.get_rect()

        skip_intro = False
        while True:
            time_passed = self.clock.tick(30)

            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.__quit()
                elif event.type == pygame.KEYDOWN:
                    #if event.key == pygame.K_m:
                    #    action = 'menu'
                    #    skip_intro = True
                    if event.key == pygame.K_q:
                        self.__quit()
            if skip_intro == True:
                break

            self.screen.blit(self.end2, bg_rect)
            pygame.display.flip()

        #if action == 'menu':
        #    self.run_without_intro()

    def win_computer(self):
        self.end1 = self.end1_image
        bg_rect = self.end1_image.get_rect()

        skip_intro = False
        while True:
            time_passed = self.clock.tick(30)

            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.__quit()
                elif event.type == pygame.KEYDOWN:
                    #if event.key == pygame.K_m:
                    #    action = 'menu'
                    #    skip_intro = True
                    if event.key == pygame.K_q:
                        self.__quit()
            if skip_intro == True:
                break

            self.screen.blit(self.end1, bg_rect)
            pygame.display.flip()

        #if action == 'menu':
        #    self.run_without_intro()

if __name__ == '__main__':
    game = Game()
    game.run2()